/*
 *
 *  Copyright (c) 1991-2020 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
 *
 * See LICENSE.TXT file for copying and redistribution conditions.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3 or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * Contact info: www.generic-mapping-tools.org
 */

#define GMT_N_COLOR_NAMES 663 /* Lines in gmt_colornames.h */
#define GMT_N_DATUMS      222 /* Lines in gmt_datums.h */
#define GMT_N_ELLIPSOIDS  76 /* Lines in gmt_ellipsoids.h */
#define GMT_N_KEYS        219 /* Lines in gmt_keywords.h */
#define GMT_N_MEDIA       31 /* Lines in gmt_media_name.h */
#define GMT_N_PEN_NAMES   12 /* Lines in gmt_pennames.h */
#define GMT_N_UNIQUE      74 /* Lines in gmt_unique.h */
#define GMT_N_CPT_MASTERS 71 /* Lines in gmt_cpt_masters.h */
#define GMT_N_DATASETS     /* Lines in gmt_datasets.h */
